/*------------------------------------------------------------------------------
#	MPing 1.1 - MacTCP Ping Tool
#
#	Copyright  Apple Computer, Inc. 1990-1991
#	All rights reserved.
#
#	Versions:	
#				1.1		September 25, 1991.
#
#	File:
#				MPing.c
#
#	Components:
#				AddressXlation.h
#				MacTCPCommonTypes.h
#				Makefile
#				MiscIPPB.h
#				MPing.c
#				MPing.h
#				MPing.r
#				MPingDlg.c
#				MPingExtern.h
#				MPingGlobals.h
#				MPingIcmp.c
#				MPingWindow.c
#				resolver.c
------------------------------------------------------------------------------*/
/* 	resolver.c - DNR library for MPW

	(c) Copyright 1988 by Apple Computer.  All rights reserved
	Jan 9, 1991.

	modified by RAB
*/

#include <Resources.h>
#include <Files.h>
#include <OSUtils.h>
#include <Memory.h>
#include <Errors.h>
#include <Traps.h>
#include <GestaltEqu.h>
#include <Folders.h>
#include <ToolUtils.h>

#define OPENRESOLVER	1
#define CLOSERESOLVER	2
#define STRTOADDR		3
#define	ADDRTOSTR		4
#define	ENUMCACHE		5
#define ADDRTONAME		6
#define	HINFO			7
#define MXINFO			8

Handle codeHndl = nil;

typedef OSErr (*OSErrProcPtr)();
OSErrProcPtr dnr = nil;


TrapType GetTrapType(theTrap)
unsigned long theTrap;
{
	if (BitAnd(theTrap, 0x0800) > 0)
		return(ToolTrap);
	else
		return(OSTrap);
}

Boolean TrapAvailable1(trap)
unsigned long trap;
{
	TrapType trapType = ToolTrap;
	unsigned long numToolBoxTraps;

	if (NGetTrapAddress(_InitGraf, ToolTrap) == NGetTrapAddress(0xAA6E, ToolTrap))
		numToolBoxTraps = 0x200;
	else
		numToolBoxTraps = 0x400;

	trapType = GetTrapType(trap);
	if (trapType = ToolTrap) {
		trap = BitAnd(trap, 0x07FF);
		if (trap > numToolBoxTraps)
			trap = _Unimplemented;
	}
	return(NGetTrapAddress(trap, trapType) != NGetTrapAddress(_Unimplemented, ToolTrap));
}

short GetCPanelFolder()
{
	short vRefNum = 0;
	long dirID = 0;
	SysEnvRec info;
	Boolean hasFolderMgr, hasGestalt = false;
	long feature;
	short wdRef;

	dirID = 0;
	if (TrapAvailable1(_GestaltDispatch)) if (Gestalt(gestaltFindFolderAttr, &feature) == noErr) hasFolderMgr = true;
	if (!hasFolderMgr) {
		SysEnvirons(1, &info);
		return(info.sysVRefNum);
	}
	else {
		if (FindFolder(kOnSystemDisk, kControlPanelFolderType, kDontCreateFolder, &vRefNum, &dirID) != noErr) return(0);
		if (OpenWD(vRefNum, dirID, 'dnrp', &wdRef) == noErr)
			return(wdRef);
		else
			return(0);
	}
}

/* OpenOurRF is called to open the MacTCP driver resources */
short OpenOurRF()
{
	ParamBlockRec fi;
	Str255 filename;
	short vRefNum;

	vRefNum = GetCPanelFolder();
	fi.fileParam.ioCompletion = nil;
	fi.fileParam.ioNamePtr = &filename;
	fi.fileParam.ioVRefNum = vRefNum;
	fi.fileParam.ioFDirIndex = 1;

	while (PBGetFInfo(&fi, false) == noErr) {
		/* scan system folder for driver resource files of specific type & creator */
		if (fi.fileParam.ioFlFndrInfo.fdType == 'cdev' &&
			fi.fileParam.ioFlFndrInfo.fdCreator == 'ztcp') {
			/* found the MacTCP driver file */
			return(OpenRFPerm(&filename, vRefNum, fsRdPerm));
		}
		/* check next file in system folder */
		fi.fileParam.ioFDirIndex++;
	}
	return(-1);
}	

OSErr OpenResolver(fileName)
char *fileName;
{
	short refnum;
	OSErr rc;
	long feature = 0;
	Boolean hasTimeMgr = false;

	if (dnr != nil)
		/* resolver already loaded in */
		return(noErr);

	/* open the MacTCP driver to get DNR resources. Search for it based on
	   creator & type rather than simply file name */	
	refnum = OpenOurRF();

	/* ignore failures since the resource may have been installed in the 
	   System file if running on a Mac 512Ke */

	/* load in the DNR resource package */
	codeHndl = GetIndResource('dnrp', 1);

	if (codeHndl == nil) {
		/* can't open DNR */
		return(ResError());
	}

	DetachResource(codeHndl);
	if (refnum != -1) {
		CloseWD(refnum);
		CloseResFile(refnum);
	}

	/* lock the DNR resource since it cannot be reloated while opened */
	HLock(codeHndl);
	dnr = (OSErrProcPtr) *codeHndl;

	/* call open resolver */
	rc = (*dnr)(OPENRESOLVER, fileName);
	if (rc != noErr) {
		/* problem with open resolver, flush it */
		HUnlock(codeHndl);
		DisposHandle(codeHndl);
		dnr = nil;
	}

	/* check is the extended time manager is available */
	if (TrapAvailable1(_GestaltDispatch)) if (Gestalt(gestaltTimeMgrVersion, &feature) == noErr) hasTimeMgr = true;
	if (hasTimeMgr) {
		if ((feature == gestaltExtendedTimeMgr) || (feature == gestaltRevisedTimeMgr))
			rc == noErr;
		else rc == 1;
	}

	return(rc);
}

OSErr CloseResolver()
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);

	/* call close resolver */
	(void) (*dnr)(CLOSERESOLVER);

	/* release the DNR resource package */
	HUnlock(codeHndl);
	DisposHandle(codeHndl);
	dnr = nil;
	return(noErr);
}

OSErr StrToAddr(hostName, rtnStruct, resultproc, userDataPtr)
char *hostName;
struct hostInfo *rtnStruct;
long resultproc;
char *userDataPtr;
{
	return((*dnr)(STRTOADDR, hostName, rtnStruct, resultproc, userDataPtr));
}

void AddrToStr(addr, addrStr)
unsigned long addr;
char *addrStr;									
{
	(*dnr)(ADDRTOSTR, addr, addrStr);
}

OSErr AddrToName(addr, rtnStruct, resultproc, userDataPtr)
unsigned long addr;
struct hostInfo *rtnStruct;
long resultproc;
char *userDataPtr;									
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);

	return((*dnr)(ADDRTONAME, addr, rtnStruct, resultproc, userDataPtr));
}
