/*
	File:		SNMPVarTypes.r

	Contains:	SNMP constants used by both code and resource files

	Copyright:	 1992 by Apple Computer, Inc., all rights reserved.

*/
/**********************************************************************
*
* SNMPVarTypes.h - SNMP Variables and their access
*
*   bit     31 = Variable is in a table (OR in this bit for all variables which are in tables)
*   bit     30 = Variables which can be accessed at interrupt time.
*   bit  24-29 = Reserved for Special Bits
*   bits 16-23 = Reserved for Display types
*   bits  8-15 = ASN Universal type
*   bits  0- 7 = Actual Tag type used in Encoding
*
***********************************************************************/
//
// Variables - Universal Types
//
#define kSNMP_Sequence	    	(0x800010FF)
#define kSNMP_SequenceOF		(0x800010FE)
#define kSNMP_Integer			(0x00000202)
#define kSNMP_OctetStr			(0x00000404)
#define kSNMP_Null	    		(0x00000505)
#define kSNMP_ObjectID			(0x00000606)

//
// Variables - Application Types
//
#define kSNMP_IPAddress			(0x00000440)    
#define kSNMP_Counter			(0x00000241) 
#define	kSNMP_Gauge				(0x00000242)
#define	kSNMP_TimeTicks			(0x00000243)
#define	kSNMP_Opaque			(0x00000444)

//
//  Variables - Macintosh Display types
//
#define	kSNMP_DisplayString		(0x00010404)
#define	kSNMP_MacDisplayString	(0x00020404)
#define	kSNMP_MacDate			(0x00030202)
#define	kSNMP_MacOS				(0x00040202)

//
// SPECIAL BITS for  Variables
//
#define	kSNMP_TableElement		(0x80000000)	// set this bit for variables in tables
#define kSNMP_IntAvail			(0x40000000)	// set this bit for variables available at interrupt time

//
// SNMP Access Modes
//
#define kSNMP_ReadOnly   0
#define kSNMP_WriteOnly  1
#define kSNMP_ReadWrite  2
#define kSNMP_NoAccess   3


//
// official object ids 
//
#define		kOID_ISO				1
#define		kOID_ORG				3
#define		kOID_DOD				6
#define		kOID_INTERNET			1
#define		kOID_MGMNT				2
#define		kOID_MIB				1
#define		kOID_SNMP				11
#define		kOID_PRIVAT				4
#define		kOID_ENTERPRISES		1
#define		kOID_APPLE				63
#define		kOID_MACINTOSH			1
#define		kOID_SYSTEM				1
#define		kOID_APPL_EXPERIMENTAL	2	
